import ecs100.*;

import java.awt.Color;

/**
 * Question 2. Design a class                       [15 marks]
 *
 * Suppose you are writing a program to simulate a production line where many boxes are
 * moving on a conveyor belt from right to left.
 *
 * Complete the Box class to specify Box objects that are used by the simulation below.
 * 
 * YOU DO NOT NEED TO MODIFY THIS CLASS. ANSWER THIS QUESTION IN THE BOX CLASS.
 */
public class Program2 {

    /**
     * Runs a simulation of a box moving from right to left on a conveyor belt.
     * You do not need to fully understand this method.
     *
     * DO NOT MODIFY THIS METHOD.
     */
    public void runSimulation() {
        // Create a new random box
        Box b = new Box((Math.random() * 200), (Math.random() * 200), Color.getHSBColor((float) Math.random(), 1.0f, 1.0f));
        while (true) {
            UI.clearGraphics();
            drawConveyorBelt();
            
            b.moveLeft(speed);  // Move the box to the left

            // If the box has moved off the edge of the screen, create a new box
            if (b.getX() < -(b.getWidth()/2)) {
                b = new Box((Math.random() * 200), (Math.random() * 200), Color.getHSBColor((float) Math.random(), 1.0f, 1.0f));
            }
            b.draw();
            UI.sleep(30);
            frame++;
        }
    }

    /*********************************************
     * YOU CAN IGNORE EVERYTHING BELOW THIS LINE *
     *********************************************/
    private int frame = 0;
    private int speed = 3;

    /*
     * Draw the conveyor belt.
     *
     * The belt is represented by two dark gray rectangles (the top and bottom) segmented with
     * black lines, and four rollers. The top of the belt moves right to left, and the bottom
     * moves left to right.
     */
    private void drawConveyorBelt() {
        // Draw the top of the belt
        UI.eraseRect(0, 300, 500,10);
        UI.setColor(Color.DARK_GRAY);
        UI.fillRect(0, 300, 500,10);
        UI.setColor(Color.BLACK);
        UI.drawRect(0, 300, 500,10);
        for(int i=0;i<5;i++) {
            // Top belt moves right to left
            int x = 500 - Math.abs((i*100+(frame*speed))%500);
            UI.drawLine(x,309,x,301);
        }
        // Draw the rollers
        drawRoller(50, 325);
        drawRoller(50+400.0/3, 325);
        drawRoller(450-400.0/3, 325);
        drawRoller(450, 325);
        // Draw the bottom of the belt
        UI.eraseRect(0, 340, 500,10);
        UI.setColor(Color.DARK_GRAY);
        UI.fillRect(0, 340, 500,10);
        UI.setColor(Color.BLACK);
        UI.drawRect(0, 340, 500,10);
        for(int i=0;i<5;i++) {
            int x = Math.abs((i*100+(frame*speed))%500);
            UI.drawLine(x,349,x,340);
        }
    }

    private void drawRoller(double x, double y) {
        UI.eraseOval(x-15,y-15, 30, 30);
        UI.setColor(Color.GRAY);
        UI.fillOval(x-15,y-15, 30, 30);
        UI.setColor(Color.DARK_GRAY);
        UI.drawOval(x-15,y-15, 30, 30);
        // In order for the animation to make sense, the outer rim of the roller
        // must move at the same (linear) speed as the belt. We then use that,
        // and the size of the roller, to determine how fast it rotates.
        // The linear velocity (v) at the radius (r) is [speed] pixels per frame.
        // The angular velocity (w) = angle (theta) / time (number of frames)
        // Ie: w = theta / t
        //     v = d / t
        //     d = r*theta
        //     v = (r*theta) / t
        //     v = r*w
        // but we only need the current angle, so:
        //    theta = (v*t)/r
        // (but negative, because we're rotating anti-clockwise)
        double theta = -(speed*frame)/15.0;
        UI.drawLine(x + 14*Math.cos(theta), y + 14*Math.sin(theta),
                    x - 14*Math.cos(theta), y - 14*Math.sin(theta));
    }


    public void setupGUI() {
        UI.initialise();
        UI.addButton("Run Simulation", this::runSimulation);
        UI.addButton("Quit", UI::quit);
        UI.setWindowSize(500, 500);
        UI.setDivider(0.0);
    }

    public static void main(String[] args) {
        new Program2().setupGUI();
    }
}
