import ecs100.*;

import java.util.ArrayList;

/**
 * Question 4. ArrayLists of Objects                     [32 Marks]
 *
 * This question concerns a program for tracking the books in a collection, as well as their
 * rating (from 0-5).

 * There are two classes: this class and the Book class. You must complete this class.
 * 
 * - This class has a 'collection' field which contains an ArrayList of Book objects.
 * - The Book class below defines Book objects, which stores the title, author, and
 *   rating of books.
 *     - The title and author are both strings, such as ”Harrow the Ninth”, or ”Tamsyn Muir”
 *     - The rating is stored as a double, but is always between 0 and 5.
 */
public class Program4 {

    private ArrayList<Book> collection = new ArrayList<Book>();

    /**
     * (a) [15 marks] Complete the searchByTitleOrAuthor(...) method that will print every
     * book that has a title or author that contains the text specified in the parameter.
     * 
     * For example, searchByTitleOrAuthor("Time") might print:
     *  - 'This Is How You Lose the Time War', by Amal El-Mohtar and Max Gladstone (5.0/5.0)
     *  - 'A History of the Brave Companions', by Timeon of Dorne (2.0/5.0)
     *  - 'A Brief History of Time', by Stephen Hawking (4.2/5)
     *  
     *  If no matching books are found, this method should display "no matching books found"
     */
    public void searchByTitleOrAuthor(String keyword) {
        /*# YOUR CODE HERE */
        boolean exists = false;
        for (Book b : collection) {
            if (b.getTitle().contains(keyword) || b.getAuthor().contains(keyword)) {
                UI.println(b.toString());
                exists = true;
            }
        }
        if (!exists) {
            UI.println("no matching books found");
        }
        /*# END OF YOUR CODE */

    }

    /**
     * (b) [17 marks]
     *   Complete the addBook(...) method so that it will create a book with the
     *   specifed title, author, and rating, and then add the book to the collection
     *   if and only if:
     *
     *  - The rating is valid (between 0 and 5 inclusive)
     *  - A book with that title is not already in the collection
     *
     * The method should return true if the book was successfully added, or false otherwise.
     */
    public boolean addBook(String title, String author, double rating) {
        /*# YOUR CODE HERE */
        if (rating < 0 || rating > 5) {
            return false;
        }
        Book b = new Book(title, author, rating);
        for (Book other : collection) {
            if (other.getTitle().equals(b.getTitle())) {
                return false;
            }
        }
        collection.add(b);
        /*# END OF YOUR CODE */

        return true;
    }

    /*********************************************
     * YOU CAN IGNORE EVERYTHING BELOW THIS LINE *
     *********************************************/
    /**
     * This method will add some data to the Schedule field so you can test
     * your code.
     */
    public void createTestData() {
        collection.clear();
        collection.add(new Book("Gideon the Ninth", "Tamsyn Muir", 4.2));
        collection.add(new Book("Harrow the Ninth", "Tamsyn Muir", 4.4));
        collection.add(new Book("This Is How You Lose the Time War", "Amal El-Mohtar and Max Gladstone", 4.0));
        collection.add(new Book("World War Z", "Max Brooks", 4.0));
        collection.add(new Book("Highway Bodies", "Alison Evans", 4.0));
        collection.add(new Book("Nimona", "Noelle Stevenson", 4.2));
        collection.add(new Book("The Witch Boy", "Molly Ostertag", 4.9));
        collection.add(new Book("A Brief History of Time", "Stephen Hawking", 4.2));
        this.printCollection();
    }

    /**
     * This method prints out the current contents of the schedule.
     * <p>
     * It is provided for your convenience.
     */
    public void printCollection() {
        UI.clearText();
        UI.println("----------------------");
        UI.println("      Collection      ");
        UI.println("----------------------");
        for (Book b : collection) {
            UI.println(b.toString());
        }
        UI.println("----------------------");
    }

    public void doAddBook() {
        String title = UI.askString("Enter title:");
        String author = UI.askString("Enter author:");
        double rating = UI.askDouble("Enter rating out of 5.0:");
        if (this.addBook(title,author,rating)) {
            UI.println("Book added!");
        } else {
            UI.println("Book not added: book conflicts with an existing book, or has an invalid rating.");
        }
    }

    public void setupGUI() {
        UI.initialise();
        UI.addButton("Create Test Data", this::createTestData);
        UI.addButton("Show Collection", this::printCollection);
        UI.addButton("searchByTitleOrAuthor", () -> {
                this.searchByTitleOrAuthor(UI.askString("Enter keyword:"));
            });
        UI.addButton("addBook", this::doAddBook);
        UI.addButton("Quit", UI::quit);
        UI.setWindowSize(700, 500);
        UI.setDivider(1.0);
    }

    public static void main(String[] args) {
        new Program4().setupGUI();
    }
}
